'use strict';

var fs        = require('fs');
var express   = require('express');
var uuid      = require('uuid');
var readChunk = require('read-chunk');
var fileType  = require('file-type');

var app = express();
var auth = {};

function parseRange(range, fileSize) {
    var parsedRange = {};

    if (range) {
        parsedRange.start = range.split('=')[1].split('-');
    } else {
        parsedRange.start = ['0', fileSize];
    }

    // Calculate requested start and end range
    parsedRange.end = parseInt(parsedRange.start[1], 10) || (fileSize - 1);
    parsedRange.start = parseInt(parsedRange.start[0], 10);

    return parsedRange;
}

app.get('/', function (req, res) {
    res.send(fs.readFileSync('test.html').toString());
});

app.get('/redirect/:resource', function (req, res) {
    var key = uuid.v4();
    var fileSize = fs.statSync(req.params.resource).size;

    auth[key] = Date.now();

    console.log(new Date().toISOString(), 'redirect: ' + req.params.resource + ',', 'key: ' + key);

    res.set('Accept-Ranges', 'bytes');
    res.set('Content-Length', fileSize);
    res.set('Cache-Control', 'no-cache, private, no-store, must-revalidate');
    res.set('Expires', 0);
    res.set('Pragma', 'no-cache');
    res.redirect(307, '/' + req.params.resource + '?key=' + key);
});

app.get('/:resource', function (req, res, next) {
    var range = {};
    var fileSize = fs.statSync(req.params.resource).size;
    var buffer = readChunk.sync(req.params.resource, 0, 262);

    console.log(new Date().toISOString(), 'resource: ' + req.params.resource + ',', 'key: ' + req.query.key);

    // console.log('--- REQ HEADERS ---');
    // console.log(JSON.stringify(req.headers, null, 2));

    if (req.query.key && ((Date.now() - auth[req.query.key]) > 1000)) {
        res.status(403);
        return next(new Error('Key expired!'));
    }

    range = parseRange(req.headers.range, fileSize);

    res.set('Content-Range', 'bytes ' + range.start + '-' + range.end + '/' + fileSize);
    res.set('Content-Length', range.end - range.start + 1);
    res.set('Content-Type', fileType(buffer).mime);
    res.set('Cache-Control', 'no-cache, private, no-store, must-revalidate, max-age=0');
    res.set('Expires', 0);
    res.set('Pragma', 'no-cache');
    res.set('Connection', 'close');
    res.status(206); // Partial content

    // console.log('--- RES HEADERS ---');
    // console.log(JSON.stringify(res._headers, null, 2));
    console.log();

    fs.createReadStream(req.params.resource, { start: range.start, end: range.end })
    .pipe(res);
});

var server = app.listen(3000, function () {
  var port = server.address().port;

  console.log(new Date().toISOString(), 'Server started http://localhost:' + port);
});
